/* Project: AVR-ASP UART console
 * Author: Andreas Butti
 * Creation Date: 2011-12-23
 * Tabsize: 4
 * Copyright: (c) 2011 Andreas Butti
 * License: GNU GPL v2
 *
 * Based on V-USB PoerSwitch example
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include <usb.h>    /* this is libusb, see http://libusb.sourceforge.net/ */

#include "aspcmd.h"

static int usbGetStringAscii(usb_dev_handle *dev, int index, int langid, char *buf, int buflen) {
	char buffer[256];
	int rval, i;

	if ((rval = usb_control_msg(dev, USB_ENDPOINT_IN, USB_REQ_GET_DESCRIPTOR, (USB_DT_STRING << 8) + index, langid, buffer, sizeof(buffer), 1000)) < 0)
		return rval;
	if (buffer[1] != USB_DT_STRING)
		return 0;
	if ((unsigned char) buffer[0] < rval)
		rval = (unsigned char) buffer[0];
	rval /= 2;
	/* lossy conversion to ISO Latin1 */
	for (i = 1; i < rval; i++) {
		if (i > buflen) /* destination buffer overflow */
			break;
		buf[i - 1] = buffer[2 * i];
		if (buffer[2 * i + 1] != 0) /* outside of ISO Latin1 range */
			buf[i - 1] = '?';
	}
	buf[i - 1] = 0;
	return i - 1;
}

static int usbOpenDevice(usb_dev_handle **device, int vendor, char *vendorName, int product, char *productName) {
	struct usb_bus *bus;
	struct usb_device *dev;
	usb_dev_handle *handle = NULL;
	int errorCode = USB_ERROR_NOTFOUND;
	static int didUsbInit = 0;

	if (!didUsbInit) {
		didUsbInit = 1;
		usb_init();
	}
	usb_find_busses();
	usb_find_devices();
	for (bus = usb_get_busses(); bus; bus = bus->next) {
		for (dev = bus->devices; dev; dev = dev->next) {
			if (dev->descriptor.idVendor == vendor && dev->descriptor.idProduct == product) {
				char string[256];
				int len;
				handle = usb_open(dev); /* we need to open the device in order to query strings */
				if (!handle) {
					errorCode = USB_ERROR_ACCESS;
					fprintf(stderr, "Warning: cannot open USB device: %s\n", usb_strerror());
					continue;
				}
				if (vendorName == NULL && productName == NULL) { /* name does not matter */
					break;
				}
				/* now check whether the names match: */
				len = usbGetStringAscii(handle, dev->descriptor.iManufacturer, 0x0409, string, sizeof(string));
				if (len < 0) {
					errorCode = USB_ERROR_IO;
					fprintf(stderr, "Warning: cannot query manufacturer for device: %s\n", usb_strerror());
				} else {
					errorCode = USB_ERROR_NOTFOUND;
					/* fprintf(stderr, "seen device from vendor ->%s<-\n", string); */
					if (strcmp(string, vendorName) == 0) {
						len = usbGetStringAscii(handle, dev->descriptor.iProduct, 0x0409, string, sizeof(string));
						if (len < 0) {
							errorCode = USB_ERROR_IO;
							fprintf(stderr, "Warning: cannot query product for device: %s\n", usb_strerror());
						} else {
							errorCode = USB_ERROR_NOTFOUND;
							/* fprintf(stderr, "seen product ->%s<-\n", string); */
							if (strcmp(string, productName) == 0)
								break;
						}
					}
				}
				usb_close(handle);
				handle = NULL;
			}
		}
		if (handle)
			break;
	}
	if (handle != NULL) {
		errorCode = 0;
		*device = handle;
	}
	return errorCode;
}

int usbasp_OpenDevice(usb_dev_handle **handle) {
	return usbOpenDevice(handle, USBDEV_SHARED_VENDOR, "www.fischl.de", USBDEV_SHARED_PRODUCT, "USBasp");
}

/*
 * wrapper for usb_control_msg call
 */
static int usbasp_transmit(usb_dev_handle *handle, unsigned char receive, unsigned char functionid, unsigned char send[4], char * buffer,
		int buffersize) {
	int nbytes;
	nbytes = usb_control_msg(handle, USB_TYPE_VENDOR | USB_RECIP_DEVICE | (receive << 7),
			functionid,
			(send[1] << 8) | send[0], (send[3] << 8) | send[2],
			buffer, buffersize, 5000);
	if (nbytes < 0) {
		fprintf(stderr, "error: usbasp_transmit: %s\n", usb_strerror());
	}

	return nbytes;
}

/**
 * Sets the baudrate of the UART
 *
 * @param baud, default 9600, maximum working @12MHz is 57600
 */
int usbasp_SetBaud(usb_dev_handle *handle, unsigned int baud) {
	int nBytes;
	unsigned char cmd[4];
	char data[8];
	cmd[0] = baud & 0xFF;
	cmd[1] = baud >> 8;

	nBytes = usbasp_transmit(handle, 0, USBASP_FUNC_UART_CONFIGURE, cmd, data, sizeof(data));

	if (nBytes < 0) {
		fprintf(stderr, "error: usbasp_transmit: %s\n", usb_strerror());
	}

	return nBytes;
}


int usbasp_SendSerial(usb_dev_handle *handle, char * buffer, int len) {
	int nBytes;
	unsigned char cmd[4];
	cmd[0] = len & 0xFF;
	cmd[1] = len >> 8;
	cmd[2] = len >> 16;
	cmd[3] = len >> 24;

	nBytes = usbasp_transmit(handle, 0, USBASP_FUNC_UART_WRITEBLOCK, cmd, buffer, len);

	if (nBytes < 0) {
		fprintf(stderr, "error: usbasp_transmit: %s\n", usb_strerror());
	}

	return nBytes;
}

/**
 * Returns the amount of bytes ready for reciving
 *
 * return the amount, or < 0 on error.
 */
int usbasp_GetSerialAvailable(usb_dev_handle *handle) {
	unsigned char buffer[8] = {0};

	int nBytes = usb_control_msg(handle, USB_TYPE_VENDOR | USB_RECIP_DEVICE | USB_ENDPOINT_IN, USBASP_FUNC_UART_AVAILABLE, 0, 0, (char *)buffer, sizeof(buffer), 5000);
	if(nBytes < 0) {
		return nBytes;
	}
	if(nBytes != 1){
		return -1;
	}

	return buffer[0];
}

/**
 * Returns the amount of bytes read
 *
 * return the amount, or < 0 on error.
 */
int usbasp_GetSerial(usb_dev_handle *handle, char *buffer, int len) {
	int nBytes;
	unsigned char cmd[4];
	cmd[0] = len & 0xFF;
	cmd[1] = len >> 8;
	cmd[2] = len >> 16;
	cmd[3] = len >> 24;

	nBytes = usbasp_transmit(handle, 1, USBASP_FUNC_UART_READBLOCK, cmd, buffer, len);

	if (nBytes < 0) {
		fprintf(stderr, "error: usbasp_transmit: %s\n", usb_strerror());
	}

	return nBytes;
}




