/* Project: AVR-ASP UART console
 * Author: Andreas Butti
 * Creation Date: 2011-12-23
 * Tabsize: 4
 * Copyright: (c) 2011 Andreas Butti
 * License: GNU GPL v2
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include <pthread.h>

#include "aspcmd.h"

usb_dev_handle *handle = NULL;
int running = 1;
pthread_t pollthread;

typedef enum {
	LINEFEED_LF,
	LINEFEED_CR,
	LINEFEED_CRLF

} LinefeedModus;

LinefeedModus newlinewModus = LINEFEED_LF;

void quit() {
	// stop polling thread
	running = 0;

	pthread_join(pollthread, NULL);

	usb_close(handle);

	printf("Quit\n");
}

void int_handler(int sig) {
	quit();
}

void *poll_function(void *ptr) {
	unsigned int available = 0;
	char buffer[512];
	int i;
	int nBytes = 0;
	int x = 0;

	while (*((int*) ptr) != 0) {
		available = usbasp_GetSerialAvailable(handle);

		if (available) {
			if (available > sizeof(buffer)) {
				available = sizeof(buffer);
			}
			nBytes = usbasp_GetSerial(handle, buffer, available);

			if (nBytes < 0) {
				fprintf(stderr, "error: reading input %i\n", nBytes);
				quit();
			}

			int lfcr_cr = 0;
			for (i = 0; i < nBytes; i++) {
				if(newlinewModus == LINEFEED_CR) {
					if (buffer[i] == '\r') {
						putc('\n', stdout);
						continue;
					} else if(buffer[i] == '\n') {
						continue;
					}
				} else if(newlinewModus == LINEFEED_CRLF) {
					if (buffer[i] == '\r') {
						lfcr_cr = 1;
						continue;
					} else if(buffer[i] == '\n') {
						if(lfcr_cr == 1) {
							putc('\n', stdout);
						}
						lfcr_cr = 0;
						continue;
					}
				}
				if (buffer[i] == '\r') {
					buffer[i] = '\n';
				}
				putc(buffer[i], stdout);
				fflush(stdout);
			}
		}

		usleep(1000 * 20);
		/**
		 * polling 50 times per second, we have a buffer of 128 bytes
		 * on the AVR, so we can theoretical handle 6'400 Bytes per second
		 *
		 * This are 9 * 6'400 = 57'600 Baud, but in practice this is not possible,
		 * because of performance problems with V-USB etc.
		 */
	}

	return 0;
}

void printHelp() {
	printf(" aspcmd [-b baud] [-n lf | cr | crlf ] \n");
	printf("   -b, --baud      specify baud rate, default: 9600\n");
	printf("   -n, --newline   [lf, cr, crlf] for newline\n");
	printf("   -h, --help      show this help\n");
	printf("\n");
}

int main(int argc, char **argv) {
	char line[512];
	int baud = 9600;
	int i;

	signal(SIGINT, int_handler);

	printf("=================================================\n");
	printf("USBAsp Serial terminal\n");
	printf("=================================================\n");

	for (i = 1; i < argc; i++) {
		if(strcmp(argv[i], "-b") == 0 || strcmp(argv[i], "--baud") == 0) {
			if(++i < argc) {
				baud = atoi(argv[i]);
			} else {
				printHelp();
				return 1;
			}
		} else if(strcmp(argv[i], "-n") == 0 || strcmp(argv[i], "--newline") == 0) {
			if(++i < argc) {
				if(strcmp(argv[i], "lf") == 0) {
					newlinewModus = LINEFEED_LF;
				} else if(strcmp(argv[i], "cr") == 0) {
					newlinewModus = LINEFEED_CR;
				} else if(strcmp(argv[i], "crlf") == 0) {
					newlinewModus = LINEFEED_CRLF;
				}

			} else {
				printHelp();
				return 1;
			}
		} else if(strcmp(argv[i], "-h") == 0 || strcmp(argv[i], "--help") == 0) {
			printHelp();
			return 0;
		} else {
			printHelp();
			return 1;
		}
	}

	usb_init();
	if (usbasp_OpenDevice(&handle) != 0) {
		fprintf(stderr, "Could not find USBAsp Device\n");
		exit(1);
	}

	usbasp_SetBaud(handle, baud);

	printf("Manual:\n");
	printf("Write a line and press <ENTER> to send\n");
	printf("Write $exit to exit\n");
	printf("\n");

	pthread_create(&pollthread, NULL, poll_function, &running);

	while (fgets(line, sizeof(line), stdin) != NULL) {
		if (strncmp(line, "$exit", 5) == 0) {
			break;
		}

		if (usbasp_SendSerial(handle, line, strlen(line)) < 0) {
			fprintf(stderr, "error writing USB device!\n");
			break;
		}

	}

	quit();

	return 0;
}

